#!/usr/bin/tclsh
# civgo.make v.2
# produces all the civgo.lc1.* and civgo.lc3.* files

# CivGo - a GUI for the Freeciv game server
# Copyright (c) 2000 by Christian Knoke

# Civgo is released under the GNU General Public License.  In short,
# you may copy this program (including source) freely, but see the
# COPYING file for full details.

set source-dir "~/freeciv-1.11.4"
set civgo-dir "~/civgo"
set locale-dir "~/civgo/civgolc"

set masterlist {tryconn server connect reconnect disconnect optset gamelevel
                created pllevel newai removed isai ishuman aination humnation
                mapkind started lisai lisbarb lishumcon lishummis lishead void1
                void2 void3 void4 save1 save0 quit accepting showhead}

set goodlist {}
set badlist {}
set meanlist {}

puts "Make lc1 and lc3 locales for Civgo"
puts "This requires the original sources for Freeciv 1.11.4"

set pofiles [glob -nocomplain "${source-dir}/po/*.po"]
if {[llength $pofiles] == 0} {
  puts "Error: Can't find the Freeciv sources!"
  exit
}

foreach filename $pofiles {
  regexp {([a-z]+(_[A-Z]+)?)\.po$} $filename all locale
  lappend locales $locale
}

puts "Will try to build locale files for the following locales:"
puts $locales
puts "loading control and master files..."

# read lc1 master strings
set f1 [open "${civgo-dir}/civgo.lc1master" r]
while {[eof $f1] == 0} {
  gets $f1 str
  if {[regexp {^([a-z]+[0-9]?) (#1 )?(#2 )?(\$[a-z]+=[^ "]+ )?(\$[a-z]+=[^ "]+ )?(\$[a-z]+=[^ "]+ )?(\$[a-z]+=[^ "]+ )?(".+")} \
      $str all name pre1 pre2 sub1 sub2 sub3 sub4 idstr]} {
    set lc1id($name) $idstr
    set subs($name,pre1) $pre1
    set subs($name,pre2) $pre2
    set subs($name,sub1) $sub1
    set subs($name,sub2) $sub2
    set subs($name,sub3) $sub3
    set subs($name,sub4) $sub4
  }
}
close $f1

# read lc3 master strings
set f1 [open "${civgo-dir}/civgo.lc3master" r]
while {[eof $f1] == 0} {
  gets $f1 str
  if {[regexp {^([a-z]+) +\+=(-?[0-9]+) +\((-?[0-9]+),(-?[0-9]+)\) +([a-zA-Z].+$)} \
      $str all name std min max text]} {
    regsub -all {"} $text {\"} text
    set lc3id($name) "\"$text\""
    set defs($name,std) $std
    set defs($name,min) $min
    set defs($name,max) $max
  }
}
close $f1

#read lc3 control data
set block none
set controllist {}
set f1 [open "${civgo-dir}/civgo.lc3control" r]
while {[eof $f1] == 0} {
  gets $f1 str
  if {[regexp {^#block:([a-z]+)} $str all block]} {
    set idx($block) 0
  }
if {[regexp {^([a-z]+) +#(live|defer) +#(scale|radio) +([0-9]+) *(\$[a-z]+)? *(\$[a-z]+)?} \
       $str all name mode type res rtxt1 rtxt2]} {
    lappend controllist $name
    set defs($name,mode) $mode
    set defs($name,type) $type
    set defs($name,res) $res
    set defs($name,block) $block
    set defs($name,rtxt1) $rtxt1
    set defs($name,rtxt2) $rtxt2
  }
}
close $f1

#lc1+lc3
proc ReadStrings locale {
  global source-dir lc1id lc1str lc3id lc3str controllist
  if {[info exists lc1str]} {unset lc1str}
  if {[info exists lc3str]} {unset lc3str}
  set f2 [open "${source-dir}/po/${locale}.po" r]
  set found 0
  while {[eof $f2] == 0} {
    gets $f2 s
    if {[regexp {^msgid +(".+"$)} $s all idstr]} {
      set found 0
      foreach name [array names lc1id] {
        if {$lc1id($name) == $idstr} {
          set found 1
          break
        }
      }
      if {$found == 0} {
        foreach name $controllist {
          if {$lc3id($name) == $idstr} {
            set found 2
            break
          }
        }
      }
    } elseif {($found > 0) && [regexp {^msgstr +(".*"$)} $s all strstr]} {
      if {$found == 1} {
        if {$strstr != "\"\""} {
          set lc1str($name) $strstr
        }
        set found 0
      }
      if {$found == 2} {
        if {$strstr != "\"\""} {
          set lc3str($name) $strstr
        }
        set found 0
      }
    }
  }
  close $f2
};#end ReadStrings

proc Buildlc1Locale locale {
  global locale-dir masterlist lc1id subs lc1str goodlist badlist meanlist
  # substitutions and parsing
  set locale-error 0
  foreach name [array names lc1str] {
    regsub -all {\(} $lc1str($name) {\\\(} lc1str($name)
    regsub -all {\)} $lc1str($name) {\\\)} lc1str($name)
    foreach pre {pre1 pre2} {
      if {$subs($name,$pre) == "#1 "} {
        regsub {[^"]+} $lc1str($name) {(&)} lc1str($name)
      }
      if {$subs($name,$pre) == "#2 "} {
        regsub {^"} $lc1str($name) {"1: } lc1str($name)
      }
    }
    foreach sub {sub1 sub2 sub3 sub4} {
      if {[regexp {(\$[a-z]+)=([^ ]+) } $subs($name,$sub) all rstr sstr]} {
        regsub {[a-z]+} $rstr "\{&\}" rstr
        if {[regsub $sstr $lc1str($name) $rstr lc1str($name)] == 0} {
          puts "Bad translation in $locale with string '$name'"
          incr locale-error 
        }
      }
    }
    regsub -all {\.} $lc1str($name) {\\\.} lc1str($name)
    regsub {^"} $lc1str($name) {"^} lc1str($name)
  }
  if {${locale-error} == 0} {
    puts "Locale $locale is good!"
  } else {
    puts "${locale-error} translation errors in locale $locale"
    puts "Locale $locale will not work right!"
  }
  # build locale file
  if {${locale-error} < 99} {
    set f3 [open "${locale-dir}/civgo.lc1.$locale" w]
    foreach name $masterlist {
      if {[info exists lc1str($name)]} {
        puts $f3 "set rel($name) $lc1str($name)"
      }
    }
    close $f3
  }
  if {${locale-error} == 0} {
    lappend goodlist $locale
  } elseif {${locale-error} < 99} {
    lappend meanlist $locale
  } else {
    lappend badlist $locale
  }
};#end Buildlc1Locale

proc Buildlc3Locale locale {
  global locale-dir lc3str controllist
  foreach block [array names idx] {set idx($block) 0}
  set f3 [open "${locale-dir}/civgo.lc3.$locale" w]
  foreach name [array names lc3str] {
    puts $f3 "set ${name}desc $lc3str($name)"
  }
  close $f3
};#end Buildlc3Locale


proc BuildWidgets {} {
  global civgo-dir lc3id lc3str defs idx controllist
  foreach block [array names idx] {set idx($block) 0}
  set f3 [open "${civgo-dir}/civgo.optionwidgets" w]
  foreach name $controllist {
    set frame $defs($name,block)
    set r [expr $idx($frame) >> 1]
    set c [expr ($idx($frame) & 1) * 3]
    if {$defs($name,type) == "scale"} {
      puts $f3 "set $name $defs($name,std)"
      puts $f3 "label .${name}txt   -text \$${name}desc -anchor e"
      puts $f3 "label .${name}val   -textvariable $name -width 5" 
      if {$defs($name,mode) == "defer"} {
        puts $f3 "scale .${name}scale -from $defs($name,min) -to $defs($name,max) \
          -resolution $defs($name,res) -showvalue 0 -variable $name -orient horizontal"
      }
      if {$defs($name,mode) == "live"} {
        puts $f3 "scale .${name}scale -from $defs($name,min) -to $defs($name,max) \
          -resolution $defs($name,res) -showvalue 0 -variable $name \
          -orient horizontal -command \"SendOption $name \""
      }
      puts $f3 "grid .${name}txt -row $r -column [expr 0 + $c] -sticky e -in .$frame"
      puts $f3 "grid .${name}val -row $r -column [expr 1 + $c] -sticky e -in .$frame"
      puts $f3 "grid .${name}scale -row $r -column [expr 2 + $c] -in .$frame"
    }
    if {$defs($name,type) == "radio"} {
      puts $f3 "set $name $defs($name,std)"
      puts $f3 "label .${name}txt -text \$${name}desc -anchor e"
      puts $f3 "frame .${name}frame"
      if {$defs($name,mode) == "defer"} {
        puts $f3 "radiobutton .${name}radio1 -value $defs($name,min) -variable $name \
          -text $defs($name,rtxt1)"
        puts $f3 "radiobutton .${name}radio2 -value $defs($name,max) -variable $name \
          -text $defs($name,rtxt2)"
      }
      if {$defs($name,mode) == "live"} {
        puts $f3 "radiobutton .${name}radio1 -value $defs($name,min) -variable $name \
          -text $defs($name,rtxt1) -command \"SendOption $name \""
        puts $f3 "radiobutton .${name}radio2 -value $defs($name,max) -variable $name \
          -text $defs($name,rtxt2) -command \"SendOption $name \""
      }
      puts $f3 "pack .${name}radio1 .${name}radio2 -in .${name}frame -side left"
      puts $f3 "grid .${name}txt -row $r -column [expr 0 + $c] -sticky e -in .$frame"
      puts $f3 "grid .${name}frame -row $r -column [expr 1 + $c] -columnspan 2 \
-in .$frame"
    }
    incr idx($frame)
  }
  close $f3
};#end BuildWidgets

puts "Building locale files - will take some time!"
puts "Building option widgets ..."
BuildWidgets
# build locale en
puts "Building regular expressions for locale en ..."
foreach name [array names lc1id] {set lc1str($name) $lc1id($name)}
Buildlc1Locale en
puts "Building option strings for locale en ..."
foreach name $controllist {set lc3str($name) $lc3id($name)}
Buildlc3Locale en

# build all other locales
foreach locale $locales {
  puts "Processing locale $locale"
  ReadStrings $locale
  Buildlc1Locale $locale
  Buildlc3Locale $locale
}

puts "### civgo.make result ###"
puts "These locales are good: $goodlist"
puts "These locales are mean: $meanlist"
puts "These locales are bad:  $badlist"

